# coding: utf-8


import pprint
import re

import six


class Identity(object):

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'issuer': 'str',
        'audience': 'str'
    }

    attribute_map = {
        'issuer': 'issuer',
        'audience': 'audience'
    }

    def __init__(self, issuer=None, audience=None):
        """Identity"""
        self._issuer = None
        self._audience = None
        self.discriminator = None
        self.issuer = issuer
        self.audience = audience

    @property
    def issuer(self):
        """Gets the issuer of this Identity.

        The issuer of the JWT token. This can be any OIDC compliant provider.

        :return: The issuer of this Identity.
        :rtype: str
        """
        return self._issuer

    @issuer.setter
    def issuer(self, issuer):
        """Sets the issuer of this Identity.

        The issuer of the JWT token. This can be any OIDC compliant provider.

        :param issuer: The issuer of this Identity.
        :type: str
        """
        if issuer is None:
            raise ValueError("Invalid value for `issuer`, must not be `None`")

        self._issuer = issuer

    @property
    def audience(self):
        """Gets the audience of this Identity.

        The target of the JWT tokens. This must be a sub target of the account app or the whole app

        :return: The audience of this Identity.
        :rtype: str
        """
        return self._audience

    @audience.setter
    def audience(self, audience):
        """Sets the audience of this Identity.

        The target of the JWT tokens. This must be a sub target of the account app or the whole app

        :param audience: The audience of this Identity.
        :type: str
        """
        if audience is None:
            raise ValueError("Invalid value for `audience`, must not be `None`")

        self._audience = audience

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(Identity, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, Identity):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8


import pprint
import re

import six


class PermissionObject(object):

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'action': 'str',
        'allow': 'bool',
        'grant': 'bool',
        'delegate': 'bool'
    }

    attribute_map = {
        'action': 'action',
        'allow': 'allow',
        'grant': 'grant',
        'delegate': 'delegate'
    }

    def __init__(self, action=None, allow=None, grant=None, delegate=None):
        """PermissionObject"""
        self._action = None
        self._allow = None
        self._grant = None
        self._delegate = None
        self.discriminator = None
        self.action = action
        self.allow = allow
        self.grant = grant
        self.delegate = delegate

    @property
    def action(self):
        """Gets the action of this PermissionObject.

        The action the permission grants, can be scoped using `:` and parent actions imply child permissions, action:* or action implies action:sub-action.

        :return: The action of this PermissionObject.
        :rtype: str
        """
        return self._action

    @action.setter
    def action(self, action):
        """Sets the action of this PermissionObject.

        The action the permission grants, can be scoped using `:` and parent actions imply child permissions, action:* or action implies action:sub-action.

        :param action: The action of this PermissionObject.
        :type: str
        """
        self._action = action

    @property
    def allow(self):
        """Gets the allow of this PermissionObject.

        Does this permission grant the user the ability to execute the action?

        :return: The allow of this PermissionObject.
        :rtype: bool
        """
        return self._allow

    @allow.setter
    def allow(self, allow):
        """Sets the allow of this PermissionObject.

        Does this permission grant the user the ability to execute the action?

        :param allow: The allow of this PermissionObject.
        :type: bool
        """
        self._allow = allow

    @property
    def grant(self):
        """Gets the grant of this PermissionObject.

        Allows the user to give the permission to others without being able to execute the action.

        :return: The grant of this PermissionObject.
        :rtype: bool
        """
        return self._grant

    @grant.setter
    def grant(self, grant):
        """Sets the grant of this PermissionObject.

        Allows the user to give the permission to others without being able to execute the action.

        :param grant: The grant of this PermissionObject.
        :type: bool
        """
        self._grant = grant

    @property
    def delegate(self):
        """Gets the delegate of this PermissionObject.

        Allows delegating or granting the permission to others without being able to execute tha action.

        :return: The delegate of this PermissionObject.
        :rtype: bool
        """
        return self._delegate

    @delegate.setter
    def delegate(self, delegate):
        """Sets the delegate of this PermissionObject.

        Allows delegating or granting the permission to others without being able to execute tha action.

        :param delegate: The delegate of this PermissionObject.
        :type: bool
        """
        self._delegate = delegate

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(PermissionObject, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, PermissionObject):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

# flake8: noqa

from __future__ import absolute_import

# import models into model package
from authress_sdk.models.access_record import AccessRecord
from authress_sdk.models.access_record_account import AccessRecordAccount
from authress_sdk.models.access_record_collection import AccessRecordCollection
from authress_sdk.models.access_record_resources import AccessRecordResources
from authress_sdk.models.access_record_statements import AccessRecordStatements
from authress_sdk.models.access_record_users import AccessRecordUsers
from authress_sdk.models.account import Account
from authress_sdk.models.account_collection import AccountCollection
from authress_sdk.models.claim_request import ClaimRequest
from authress_sdk.models.claim_response import ClaimResponse
from authress_sdk.models.client import Client
from authress_sdk.models.client_access_key import ClientAccessKey
from authress_sdk.models.client_collection import ClientCollection
from authress_sdk.models.client_options import ClientOptions
from authress_sdk.models.identity import Identity
from authress_sdk.models.identity_collection import IdentityCollection
from authress_sdk.models.identity_request import IdentityRequest
from authress_sdk.models.user_resources import UserResources
from authress_sdk.models.user_permissions import UserPermissions
from authress_sdk.models.permission_object import PermissionObject
from authress_sdk.models.resource_permission import ResourcePermission
from authress_sdk.models.resource_permission_collection import ResourcePermissionCollection
from authress_sdk.models.resource_permission_permissions import ResourcePermissionPermissions



# coding: utf-8


from __future__ import absolute_import

import re

# python 2 and python 3 compatibility library
import six

from authress_sdk.api_client import ApiClient


class ServiceClientsApi(object):
    def __init__(self, api_client=None):
        if api_client is None:
            api_client = ApiClient()
        self.api_client = api_client

    def delete_access_key(self, client_id, key_id, **kwargs):
        """Remove an access key for a client

        Deletes an access key for a client prevent it from being used to authenticate with Authress.
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.delete_access_key(client_id, key_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str client_id: The unique identifier of the client. (required)
        :param str key_id: The id of the access key to remove from the client. (required)
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.delete_access_key_with_http_info(client_id, key_id, **kwargs)
        else:
            (data) = self.delete_access_key_with_http_info(client_id, key_id, **kwargs)
            return data

    def delete_access_key_with_http_info(self, client_id, key_id, **kwargs):
        """Remove an access key for a client

        Deletes an access key for a client prevent it from being used to authenticate with Authress.
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.delete_access_key_with_http_info(client_id, key_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str client_id: The unique identifier of the client. (required)
        :param str key_id: The id of the access key to remove from the client. (required)
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['client_id', 'key_id']
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method delete_access_key" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'client_id' is set
        if ('client_id' not in params or
                params['client_id'] is None):
            raise ValueError("Missing the required parameter `client_id`.")
        # verify the required parameter 'key_id' is set
        if ('key_id' not in params or
                params['key_id'] is None):
            raise ValueError("Missing the required parameter `key_id`.")

        collection_formats = {}

        path_params = {}
        if 'client_id' in params:
            path_params['clientId'] = params['client_id']
        if 'key_id' in params:
            path_params['keyId'] = params['key_id']

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None


        return self.api_client.call_api(
            '/v1/clients/{clientId}/access-keys/{keyId}', 'DELETE',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type=None,

            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def request_access_key(self, client_id, **kwargs):
        """Request a new access key

        Create a new access key for the client so that a service can authenticate with Authress as that client. Using the client will allow delegation of permission checking of users.
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.request_access_key(client_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str client_id: The unique identifier of the client. (required)
        :return: ClientAccessKey
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.request_access_key_with_http_info(client_id, **kwargs)
        else:
            (data) = self.request_access_key_with_http_info(client_id, **kwargs)
            return data

    def request_access_key_with_http_info(self, client_id, **kwargs):
        """Request a new access key

        Create a new access key for the client so that a service can authenticate with Authress as that client. Using the client will allow delegation of permission checking of users.
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.request_access_key_with_http_info(client_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str client_id: The unique identifier of the client. (required)
        :return: ClientAccessKey
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['client_id']
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method request_access_key" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'client_id' is set
        if ('client_id' not in params or
                params['client_id'] is None):
            raise ValueError("Missing the required parameter `client_id`.")

        collection_formats = {}

        path_params = {}
        if 'client_id' in params:
            path_params['clientId'] = params['client_id']

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/links+json'])



        return self.api_client.call_api(
            '/v1/clients/{clientId}/access-keys', 'POST',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='ClientAccessKey',

            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def delete_client(self, client_id, **kwargs):
        """Delete a client

        This deletes the service client.
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.delete_client(client_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str client_id: The unique identifier for the client. (required)
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.delete_client_with_http_info(client_id, **kwargs)
        else:
            (data) = self.delete_client_with_http_info(client_id, **kwargs)
            return data

    def delete_client_with_http_info(self, client_id, **kwargs):
        """Delete a client

        This deletes the service client.
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.delete_client_with_http_info(client_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str client_id: The unique identifier for the client. (required)
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['client_id']
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method v1_clients_client_id_delete" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'client_id' is set
        if ('client_id' not in params or
                params['client_id'] is None):
            raise ValueError("Missing the required parameter `client_id`.")

        collection_formats = {}

        path_params = {}
        if 'client_id' in params:
            path_params['clientId'] = params['client_id']

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None


        return self.api_client.call_api(
            '/v1/clients/{clientId}', 'DELETE',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type=None,

            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def get_client(self, client_id, **kwargs):
        """Get a client.

        Returns all information related to client except for the private access keys.
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_client(client_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str client_id: The unique identifier for the client. (required)
        :return: Client
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.get_client_with_http_info(client_id, **kwargs)
        else:
            (data) = self.get_client_with_http_info(client_id, **kwargs)
            return data

    def get_client_with_http_info(self, client_id, **kwargs):
        """Get a client.

        Returns all information related to client except for the private access keys.
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_client_with_http_info(client_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str client_id: The unique identifier for the client. (required)
        :return: Client
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['client_id']
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method get_client" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'client_id' is set
        if ('client_id' not in params or
                params['client_id'] is None):
            raise ValueError("Missing the required parameter `client_id`.")

        collection_formats = {}

        path_params = {}
        if 'client_id' in params:
            path_params['clientId'] = params['client_id']

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/links+json'])



        return self.api_client.call_api(
            '/v1/clients/{clientId}', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='Client',

            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def update_client(self, body, client_id, **kwargs):
        """Update a client

        Updates a client information.
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.update_client(body, client_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param Client body: (required)
        :param str client_id: The unique identifier for the client. (required)
        :return: Client
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.update_client_http_info(body, client_id, **kwargs)
        else:
            (data) = self.update_client_http_info(body, client_id, **kwargs)
            return data

    def update_client_http_info(self, body, client_id, **kwargs):
        """Update a client

        Updates a client information.
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.update_client_http_info(body, client_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param Client body: (required)
        :param str client_id: The unique identifier for the client. (required)
        :return: Client
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['body', 'client_id']
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method v1_clients_client_id_put" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'body' is set
        if ('body' not in params or
                params['body'] is None):
            raise ValueError("Missing the required parameter `body`.")
        # verify the required parameter 'client_id' is set
        if ('client_id' not in params or
                params['client_id'] is None):
            raise ValueError("Missing the required parameter `client_id`.")

        collection_formats = {}

        path_params = {}
        if 'client_id' in params:
            path_params['clientId'] = params['client_id']

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        if 'body' in params:
            body_params = params['body']
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/links+json'])

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(
            ['application/json'])



        return self.api_client.call_api(
            '/v1/clients/{clientId}', 'PUT',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='Client',

            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def get_clients(self, **kwargs):
        """Get clients collection

        Returns all clients that the user has access to in the account.
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_clients(async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :return: ClientCollection
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.get_clients_with_http_info(**kwargs)
        else:
            (data) = self.get_clients_with_http_info(**kwargs)
            return data

    def get_clients_with_http_info(self, **kwargs):
        """Get clients collection

        Returns all clients that the user has access to in the account.
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_clients_with_http_info(async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :return: ClientCollection
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = []
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method get_clients" % key
                )
            params[key] = val
        del params['kwargs']

        collection_formats = {}

        path_params = {}

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/links+json'])



        return self.api_client.call_api(
            '/v1/clients', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='ClientCollection',

            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def create_client(self, body, **kwargs):
        """Create a new client.

        Creates a service client to interact with Authress or any other service on behalf of users. Each client has secret private keys used to authenticate with Authress. To use service clients created through other mechanisms, skip creating a client and create access records with the client identifier.
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.create_client(body, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param Client body: (required)
        :return: Client
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.create_client_with_http_info(body, **kwargs)
        else:
            (data) = self.create_client_with_http_info(body, **kwargs)
            return data

    def create_client_with_http_info(self, body, **kwargs):
        """Create a new client.

        Creates a service client to interact with Authress or any other service on behalf of users. Each client has secret private keys used to authenticate with Authress. To use service clients created through other mechanisms, skip creating a client and create access records with the client identifier.
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.create_client_with_http_info(body, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param Client body: (required)
        :return: Client
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['body']
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method create_client" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'body' is set
        if ('body' not in params or
                params['body'] is None):
            raise ValueError("Missing the required parameter `body`.")

        collection_formats = {}

        path_params = {}

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        if 'body' in params:
            body_params = params['body']
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/links+json'])

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(
            ['application/json'])



        return self.api_client.call_api(
            '/v1/clients', 'POST',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='Client',

            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)



# coding: utf-8


import pprint
import re

import six


class Account(object):

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'account_id': 'str',
        'created_time': 'datetime',
        'domain': 'str',
        'company': 'object',
        'links': 'object'
    }

    attribute_map = {
        'account_id': 'accountId',
        'created_time': 'createdTime',
        'domain': 'domain',
        'company': 'company',
        'links': 'links'
    }

    def __init__(self, account_id=None, created_time=None, domain=None, company=None, links=None):
        """Account"""
        self._account_id = None
        self._created_time = None
        self._domain = None
        self._company = None
        self._links = None
        self.discriminator = None
        self.account_id = account_id
        self.created_time = created_time
        self.domain = domain
        self.company = company
        self.links = links

    @property
    def account_id(self):
        """Gets the account_id of this Account.


        :return: The account_id of this Account.
        :rtype: str
        """
        return self._account_id

    @account_id.setter
    def account_id(self, account_id):
        """Sets the account_id of this Account.


        :param account_id: The account_id of this Account.
        :type: str
        """
        self._account_id = account_id

    @property
    def created_time(self):
        """Gets the created_time of this Account.


        :return: The created_time of this Account.
        :rtype: datetime
        """
        return self._created_time

    @created_time.setter
    def created_time(self, created_time):
        """Sets the created_time of this Account.


        :param created_time: The created_time of this Account.
        :type: datetime
        """
        self._created_time = created_time

    @property
    def domain(self):
        """Gets the domain of this Account.

        The top authress sub domain specific for this account to be used with this API.

        :return: The domain of this Account.
        :rtype: str
        """
        return self._domain

    @domain.setter
    def domain(self, domain):
        """Sets the domain of this Account.

        The top authress sub domain specific for this account to be used with this API.

        :param domain: The domain of this Account.
        :type: str
        """
        self._domain = domain

    @property
    def company(self):
        """Gets the company of this Account.


        :return: The company of this Account.
        :rtype: object
        """
        return self._company

    @company.setter
    def company(self, company):
        """Sets the company of this Account.


        :param company: The company of this Account.
        :type: object
        """
        self._company = company

    @property
    def links(self):
        """Gets the links of this Account.


        :return: The links of this Account.
        :rtype: object
        """
        return self._links

    @links.setter
    def links(self, links):
        """Sets the links of this Account.


        :param links: The links of this Account.
        :type: object
        """
        self._links = links

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(Account, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, Account):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8


import pprint
import re

import six


class Client(object):

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'client_id': 'str',
        'created_time': 'datetime',
        'name': 'str',
        'options': 'ClientOptions'
    }

    attribute_map = {
        'client_id': 'clientId',
        'created_time': 'createdTime',
        'name': 'name',
        'options': 'options'
    }

    def __init__(self, client_id=None, created_time=None, name=None, options=None):
        """Client"""
        self._client_id = None
        self._created_time = None
        self._name = None
        self._options = None
        self.discriminator = None
        self.client_id = client_id
        self.created_time = created_time
        self.name = name
        if options is not None:
            self.options = options

    @property
    def client_id(self):
        """Gets the client_id of this Client.

        The unique id of the client.

        :return: The client_id of this Client.
        :rtype: str
        """
        return self._client_id

    @client_id.setter
    def client_id(self, client_id):
        """Sets the client_id of this Client.

        The unique id of the client.

        :param client_id: The client_id of this Client.
        :type: str
        """
        self._client_id = client_id

    @property
    def created_time(self):
        """Gets the created_time of this Client.


        :return: The created_time of this Client.
        :rtype: datetime
        """
        return self._created_time

    @created_time.setter
    def created_time(self, created_time):
        """Sets the created_time of this Client.


        :param created_time: The created_time of this Client.
        :type: datetime
        """
        self._created_time = created_time

    @property
    def name(self):
        """Gets the name of this Client.

        The name of the client

        :return: The name of this Client.
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this Client.

        The name of the client

        :param name: The name of this Client.
        :type: str
        """
        self._name = name

    @property
    def options(self):
        """Gets the options of this Client.


        :return: The options of this Client.
        :rtype: ClientOptions
        """
        return self._options

    @options.setter
    def options(self, options):
        """Sets the options of this Client.


        :param options: The options of this Client.
        :type: ClientOptions
        """

        self._options = options

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(Client, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, Client):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8


import pprint
import re

import six


class AccessRecord(object):

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'record_id': 'str',
        'name': 'str',
        'account': 'AccessRecordAccount',
        'users': 'list[AccessRecordUsers]',
        'admins': 'list[AccessRecordUsers]',
        'statements': 'list[AccessRecordStatements]',
        'links': 'object'
    }

    attribute_map = {
        'record_id': 'recordId',
        'name': 'name',
        'account': 'account',
        'users': 'users',
        'admins': 'admins',
        'statements': 'statements',
        'links': 'links'
    }

    def __init__(self, record_id=None, name=None, account=None, users=None, admins=None, statements=None, links=None):
        """AccessRecord"""
        self._record_id = None
        self._name = None
        self._account = None
        self._users = None
        self._admins = None
        self._statements = None
        self._links = None
        self.discriminator = None
        self.record_id = record_id
        self.name = name
        self.account = account
        self.users = users
        self.admins = admins
        self.statements = statements
        self.links = links

    @property
    def record_id(self):
        """Gets the record_id of this AccessRecord.

        Unique identifier for the record, can be specified on record creation.

        :return: The record_id of this AccessRecord.
        :rtype: str
        """
        return self._record_id

    @record_id.setter
    def record_id(self, record_id):
        """Sets the record_id of this AccessRecord.

        Unique identifier for the record, can be specified on record creation.

        :param record_id: The record_id of this AccessRecord.
        :type: str
        """
        self._record_id = record_id

    @property
    def name(self):
        """Gets the name of this AccessRecord.

        A helpful name for this record

        :return: The name of this AccessRecord.
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this AccessRecord.

        A helpful name for this record

        :param name: The name of this AccessRecord.
        :type: str
        """
        self._name = name

    @property
    def account(self):
        """Gets the account of this AccessRecord.


        :return: The account of this AccessRecord.
        :rtype: AccessRecordAccount
        """
        return self._account

    @account.setter
    def account(self, account):
        """Sets the account of this AccessRecord.


        :param account: The account of this AccessRecord.
        :type: AccessRecordAccount
        """
        self._account = account

    @property
    def users(self):
        """Gets the users of this AccessRecord.

        The list of users this record applies to

        :return: The users of this AccessRecord.
        :rtype: list[AccessRecordUsers]
        """
        return self._users

    @users.setter
    def users(self, users):
        """Sets the users of this AccessRecord.

        The list of users this record applies to

        :param users: The users of this AccessRecord.
        :type: list[AccessRecordUsers]
        """
        self._users = users

    @property
    def admins(self):
        """Gets the admins of this AccessRecord.

        The list of admin that can edit this record even if they do not have global record edit permissions.

        :return: The admins of this AccessRecord.
        :rtype: list[AccessRecordUsers]
        """
        return self._admins

    @admins.setter
    def admins(self, admins):
        """Sets the admins of this AccessRecord.

        The list of admin that can edit this record even if they do not have global record edit permissions.

        :param admins: The admins of this AccessRecord.
        :type: list[AccessRecordUsers]
        """
        self._admins = admins

    @property
    def statements(self):
        """Gets the statements of this AccessRecord.

        A list of statements which match roles to resources. Users in this record have all statements apply to them

        :return: The statements of this AccessRecord.
        :rtype: list[AccessRecordStatements]
        """
        return self._statements

    @statements.setter
    def statements(self, statements):
        """Sets the statements of this AccessRecord.

        A list of statements which match roles to resources. Users in this record have all statements apply to them

        :param statements: The statements of this AccessRecord.
        :type: list[AccessRecordStatements]
        """
        if statements is None:
            raise ValueError("Invalid value for `statements`, must not be `None`")

        self._statements = statements

    @property
    def links(self):
        """Gets the links of this AccessRecord.


        :return: The links of this AccessRecord.
        :rtype: object
        """
        return self._links

    @links.setter
    def links(self, links):
        """Sets the links of this AccessRecord.


        :param links: The links of this AccessRecord.
        :type: object
        """
        self._links = links

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(AccessRecord, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, AccessRecord):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other
